#include "gloobus-preview-interface-main.h"

// ================================= SET FILE NAME ============================ //
// @param filename is the path to the file (not URI)
// @returns void
//
// It sets the member variable filename, gfile and sets the basic information
void interface::set_filename	( string filename )	
{
	m_filename = filename; 
	m_gfile    = g_file_new_for_path (filename.c_str () );

	set_file_info();
	

}
// ================================= SET FILE INFO ============================ //
void interface::set_file_info ()
{

	GFileInfo * info = g_file_info_new	 ();
	info 			 = g_file_query_info ( m_gfile,  "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);

	m_filetype 		 =  g_file_info_get_attribute_string 	( info, "standard::content-type" );

	g_debug("\033[1;37mFILE INFO\033[1;m\n");
	g_debug("Screen size: %ix%i\n",	get_screen_width(),get_screen_height());
	g_debug("URI:   %s \n", 	 	g_file_get_uri ( m_gfile));
	g_debug("THUMB: %s \n",    		g_file_info_get_attribute_byte_string 			( info, "thumbnail::path" ));
	g_debug("Type:  %s \n" ,   		m_filetype.c_str());
	g_debug("Size:  %s \n", 		get_file_size(m_filename).c_str());
}


// ================================= GET SAFE TYPE ============================ //
string interface::get_safe_type	(bool replace_slash )
{
	string mime = m_filetype;
	mime = mime.substr(0,mime.find(";"));
	mime = mime.substr(0,mime.find(","));

	if(mime == "x-directory/normal")		{ mime = "folder";}

	if(replace_slash)
	{
		if(mime.find("/") != string::npos)
			mime.replace(mime.find("/"),1,"-");
	}

	return mime;
}
// ================================ GET FILE SIZE ============================== //
string interface::get_file_size	( string filename)
{
	//g_print("Getting Folder Size (NAME): %s\n", filename.c_str());
	
	GFile 	  * file = g_file_new_for_path 				( filename.c_str() );
	GFileInfo * info = g_file_info_new	 				( );
	info 			 = g_file_query_info 				( file,  "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);
	string mime	 	 = g_file_info_get_attribute_string ( info, "standard::content-type" );
	
	string units = "bytes";
	char buff[10];
	double size;
    
    size = (double)g_file_info_get_attribute_uint64 ( info, "standard::size" );
    //g_debug("MIMEEEE: %s",mime.c_str());
    if(mime== "inode/directory")
    {
    	size = get_directory_size(filename);
    }
        
    if(size > 1024)
    { 
    	size /=1024.0;
    	units = "KB";
    }
    if(size > 1024)
    { 
    	size /= 1024.0;
    	units = "MB";
    }
    if(size > 1024)
    { 
    	size /= 1024.0;
    	units = "GB";
    }
    
	sprintf(buff,"%.2f ",size);
	return buff + units;


}
// ================================ GET FILE SIZE ============================== //
double interface::get_file_size	( string filename, bool number)
{
	
	GFile 	  * file = g_file_new_for_path 				( filename.c_str() );
	GFileInfo * info = g_file_info_new	 				( );
	info 			 = g_file_query_info 				( file,  "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);
	string mime	 	 = g_file_info_get_attribute_string ( info, "standard::content-type" );
	
	double size;
    
    size = g_file_info_get_attribute_uint64 ( info, "standard::size" );
    if(mime== "inode/directory")
    {
    	size = get_directory_size(filename);
    }
 
	return size;

}
// ================================= GET DIRECTORY SIZE ======================== //
double interface::get_directory_size ( string filename )
{
 
    static unsigned int BUF_SIZE = 1024;
	FILE *dl;					// File Handle
	char in_buf[BUF_SIZE]; 		// input buffer for lib
    
   //g_debug("Loading folder size: %s \n", m_filename.c_str());
	string cmd = "du \"" + filename + "\" -sL | cut -f1";		//du <dir> -h -s | cut -f1
	
	dl = popen(cmd.c_str(), "r");		//List all plugins (*.so);
   	if(!dl){      
		perror("popen");      
		exit(-1);   	
	}

	fgets(in_buf, BUF_SIZE, dl);
	//g_debug("FOLDER SIZE============>: %i\n",atoi(in_buf)*1024);
	
	return atoi(in_buf)*1024;
}
// ================================= DRAW FILE INFO ============================ //
void interface::draw_file_info( GtkContainer * container)
{
	draw_properties(container , get_file_info_string (), 20 + SHADOW_WIDTH , get_height() + SHADOW_WIDTH - 20);
}

// ================================= DRAW PROPERTIES ============================ //
void interface::draw_properties( GtkContainer * container , string text, int x, int y)
{

	PangoAttrList* at = pango_attr_list_new();
	pango_attr_list_insert(at,pango_attr_foreground_new(65000,65000,65000));
	pango_attr_list_insert(at,pango_attr_weight_new(PANGO_WEIGHT_NORMAL));

	//GtkWidget * label = gtk_label_new(text.c_str());
	GtkWidget * label = gtk_label_new(text.c_str());
	
						g_debug("MAX LETTERS : %i",(get_width()-SHADOW_WIDTH*2)/9);
			            gtk_label_set_max_width_chars (GTK_LABEL(label), (get_width()-SHADOW_WIDTH*2)/9 );
	gtk_label_set_attributes(GTK_LABEL(label),at);

	GtkWidget *label_container = gtk_fixed_new ();
	gtk_fixed_put(GTK_FIXED(label_container),label, x ,y);
	gtk_container_add (GTK_CONTAINER(container),label_container);

	gtk_widget_show	  (label);
	gtk_widget_show	  (label_container);
}

// ================================= GET FILE INFO STRING ====================== //
string interface::get_file_info_string ( void )
{

	GFileInfo * info = g_file_info_new	 ();
	info 			 = g_file_query_info ( m_gfile,  "*" ,G_FILE_QUERY_INFO_NONE, NULL , NULL);

	string text;
	
	text = "";
	text = text + _("Type: ") + g_file_info_get_attribute_string ( info, "standard::content-type" ) + "\n\n";
	text = text + _("Size: ") + get_file_size(m_filename) + "\n";
	
	return text;

}

// ================================= GET SCREEN SIZE ============================ //
int interface::get_screen_width ( void )
{
	GdkScreen *screen = gdk_screen_get_default ();
	return gdk_screen_get_width(screen);
}
int interface::get_screen_height( void )
{
	GdkScreen *screen = gdk_screen_get_default ();
	return gdk_screen_get_height(screen);
}
//==================================== PIXBUF FUNCTIONS ===========================================//
//==================================== PIXBUF FUNCTIONS ===========================================//
//==================================== PIXBUF FUNCTIONS ===========================================//

// ================================= GET ICON ============================ //
GdkPixbuf * interface::get_icon ( void )
{

    GIcon *icon;
    GtkIconInfo *icon_info;
    GFileInfo *file_info;
    GtkIconTheme *icon_theme;

    icon_theme = gtk_icon_theme_get_default();
	file_info = g_file_query_info(
                    m_gfile,
                    G_FILE_ATTRIBUTE_STANDARD_ICON "," G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
                    G_FILE_QUERY_INFO_NONE, NULL , NULL);
    icon = g_file_info_get_icon(file_info);
    icon_info = gtk_icon_theme_lookup_by_gicon(
                    icon_theme,
                    icon,
                    200,    /* icon size */
                   GtkIconLookupFlags(GTK_ICON_LOOKUP_USE_BUILTIN | 
                   					  GTK_ICON_LOOKUP_GENERIC_FALLBACK | 
                   					  GTK_ICON_LOOKUP_FORCE_SIZE));

   return gtk_icon_info_load_icon(icon_info, NULL);
   
}


//============================== CREATE REFLECTION ========================= //
GdkPixbuf * interface::pixbuf_reflection	( GdkPixbuf * pixbuf )
{
	/*
	REFLECTION

	 0 = transparent, 255=opac
	 
		  Fliped Pixbuf
		    0 --------------  initial_opacity
		    1 --------------  initial_opacity - (1*constant)
		    2 --------------  initial_opacity - (2*constant)
		    3 --------------  initial_opacity - (3*constant)
		    4 --------------  initial_opacity - (4*constant)
	*/ 
	GdkPixbuf * temp;
	temp = gdk_pixbuf_flip	(pixbuf,false);
	int x 	= gdk_pixbuf_get_width	(temp);
	int y 	= gdk_pixbuf_get_height	(temp);
	int initial_opacity = 77;

	int width, height, rowstride, n_channels;
	guchar *pixels, *p;
	n_channels = gdk_pixbuf_get_n_channels 		( temp );
	g_assert (gdk_pixbuf_get_colorspace 		( temp ) == GDK_COLORSPACE_RGB );
	g_assert (gdk_pixbuf_get_bits_per_sample 	( temp ) == 8);
	g_assert (gdk_pixbuf_get_has_alpha 			( temp ));
	g_assert (n_channels == 4);
	width = gdk_pixbuf_get_width 				( temp );
	height = gdk_pixbuf_get_height 				( temp );
	//g_assert (x >= 0 && x < width);
	//g_assert (y >= 0 && y < height);
	rowstride 	= gdk_pixbuf_get_rowstride 		( temp );
	pixels 		= gdk_pixbuf_get_pixels 		( temp );
	//p 		= pixels + y * rowstride + x * n_channels;



	for(int i=0; i<x; i++)
	{
		for(int j=0; j<y; j++)
		{
			//TODO: Add the set_opacity fucntions here!!
			int less = j*1.3;
			if (less > initial_opacity) less = initial_opacity;
			//set_pixel_opacity(temp,i,j,initial_opacity-less);
			p = pixels + j * rowstride + i * n_channels;
			guchar temp = 255 - (initial_opacity-less);				//initial_opacity-less = alpha
			guchar temp2 = p[3] - temp;
	
			if		(temp >= p[3]) { p[3] = 0;		 }
			else		 		   { p[3] = p[3] - temp;  }
		
		}	
	}

	return temp;
}

// ========================== SCALE PIXBUF to 300x300 max ================================= //
// Scale the image to the maxium size of max_x and max_y
GdkPixbuf* interface::scale_pixbuf(GdkPixbuf * pixbuf, int max_x, int max_y)
{
	g_debug("Scaling pixbuf to %ix%i\n",max_x,max_y);
	
	int x = gdk_pixbuf_get_width  (pixbuf);
	int y = gdk_pixbuf_get_height (pixbuf);

	bool modify = false;
	double value;
	
	if(x > max_x)
	{
		value = max_x/float(x);
		x = x*value;
		y = y*value;
		modify=true;
	}	
	if(y > max_y)  
	{
		value = max_y/float(y);
		x = x*value;
		y = y*value;
		modify=true;
	}

	if(modify)
	{
		value = double(x)/(gdk_pixbuf_get_width(pixbuf));

		g_debug("Scaling Image (%f):%dx%d\n",value,gdk_pixbuf_get_width(pixbuf),gdk_pixbuf_get_height(pixbuf));
		g_debug("--> %ix%i\n",x,y);
	}

	GdkPixbuf * newPixbuf;
	newPixbuf  = gdk_pixbuf_scale_simple (pixbuf,x,y,GDK_INTERP_BILINEAR);
	gdk_pixbuf_unref (pixbuf);
	newPixbuf = gdk_pixbuf_add_alpha(newPixbuf,false,0,0,0);			//Add Alpha Channel
	return newPixbuf;

}




